from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.metric import Metric
    from ..models.qps import QPS
    from ..models.resource_environment_metrics_inference_per_second_per_region import (
        ResourceEnvironmentMetricsInferencePerSecondPerRegion,
    )
    from ..models.resource_environment_metrics_query_per_second_per_region_per_code import (
        ResourceEnvironmentMetricsQueryPerSecondPerRegionPerCode,
    )


T = TypeVar("T", bound="ResourceEnvironmentMetrics")


@_attrs_define
class ResourceEnvironmentMetrics:
    """Metrics for a single resource deployment (eg. model deployment, function deployment)

    Attributes:
        inference_per_second_global (Union[Unset, List['Metric']]): Array of metrics
        inference_per_second_per_region (Union[Unset, ResourceEnvironmentMetricsInferencePerSecondPerRegion]):
            Historical requests per second (RPS) per location, for the model deployment
        query_per_second_global (Union[Unset, float]): RPS value (in last 24 hours) for the model deployment globally
        query_per_second_per_code_global (Union[Unset, QPS]): Query per second per element, can be per response status
            code (e.g. 200, 400) or per location
        query_per_second_per_region (Union[Unset, QPS]): Query per second per element, can be per response status code
            (e.g. 200, 400) or per location
        query_per_second_per_region_per_code (Union[Unset, ResourceEnvironmentMetricsQueryPerSecondPerRegionPerCode]):
            RPS value (in last 24 hours) per response status code per location, for the model deployment
    """

    inference_per_second_global: Union[Unset, List["Metric"]] = UNSET
    inference_per_second_per_region: Union[
        Unset, "ResourceEnvironmentMetricsInferencePerSecondPerRegion"
    ] = UNSET
    query_per_second_global: Union[Unset, float] = UNSET
    query_per_second_per_code_global: Union[Unset, "QPS"] = UNSET
    query_per_second_per_region: Union[Unset, "QPS"] = UNSET
    query_per_second_per_region_per_code: Union[
        Unset, "ResourceEnvironmentMetricsQueryPerSecondPerRegionPerCode"
    ] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        inference_per_second_global: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.inference_per_second_global, Unset):
            inference_per_second_global = []
            for (
                componentsschemas_array_metric_item_data
            ) in self.inference_per_second_global:
                componentsschemas_array_metric_item = (
                    componentsschemas_array_metric_item_data.to_dict()
                )
                inference_per_second_global.append(componentsschemas_array_metric_item)

        inference_per_second_per_region: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.inference_per_second_per_region, Unset):
            inference_per_second_per_region = (
                self.inference_per_second_per_region.to_dict()
            )

        query_per_second_global = self.query_per_second_global

        query_per_second_per_code_global: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.query_per_second_per_code_global, Unset):
            query_per_second_per_code_global = (
                self.query_per_second_per_code_global.to_dict()
            )

        query_per_second_per_region: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.query_per_second_per_region, Unset):
            query_per_second_per_region = self.query_per_second_per_region.to_dict()

        query_per_second_per_region_per_code: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.query_per_second_per_region_per_code, Unset):
            query_per_second_per_region_per_code = (
                self.query_per_second_per_region_per_code.to_dict()
            )

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if inference_per_second_global is not UNSET:
            field_dict["inference_per_second_global"] = inference_per_second_global
        if inference_per_second_per_region is not UNSET:
            field_dict["inference_per_second_per_region"] = (
                inference_per_second_per_region
            )
        if query_per_second_global is not UNSET:
            field_dict["query_per_second_global"] = query_per_second_global
        if query_per_second_per_code_global is not UNSET:
            field_dict["query_per_second_per_code_global"] = (
                query_per_second_per_code_global
            )
        if query_per_second_per_region is not UNSET:
            field_dict["query_per_second_per_region"] = query_per_second_per_region
        if query_per_second_per_region_per_code is not UNSET:
            field_dict["query_per_second_per_region_per_code"] = (
                query_per_second_per_region_per_code
            )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.metric import Metric
        from ..models.qps import QPS
        from ..models.resource_environment_metrics_inference_per_second_per_region import (
            ResourceEnvironmentMetricsInferencePerSecondPerRegion,
        )
        from ..models.resource_environment_metrics_query_per_second_per_region_per_code import (
            ResourceEnvironmentMetricsQueryPerSecondPerRegionPerCode,
        )

        d = src_dict.copy()
        inference_per_second_global = []
        _inference_per_second_global = d.pop("inference_per_second_global", UNSET)
        for componentsschemas_array_metric_item_data in (
            _inference_per_second_global or []
        ):
            componentsschemas_array_metric_item = Metric.from_dict(
                componentsschemas_array_metric_item_data
            )

            inference_per_second_global.append(componentsschemas_array_metric_item)

        _inference_per_second_per_region = d.pop(
            "inference_per_second_per_region", UNSET
        )
        inference_per_second_per_region: Union[
            Unset, ResourceEnvironmentMetricsInferencePerSecondPerRegion
        ]
        if isinstance(_inference_per_second_per_region, Unset):
            inference_per_second_per_region = UNSET
        else:
            inference_per_second_per_region = (
                ResourceEnvironmentMetricsInferencePerSecondPerRegion.from_dict(
                    _inference_per_second_per_region
                )
            )

        query_per_second_global = d.pop("query_per_second_global", UNSET)

        _query_per_second_per_code_global = d.pop(
            "query_per_second_per_code_global", UNSET
        )
        query_per_second_per_code_global: Union[Unset, QPS]
        if isinstance(_query_per_second_per_code_global, Unset):
            query_per_second_per_code_global = UNSET
        else:
            query_per_second_per_code_global = QPS.from_dict(
                _query_per_second_per_code_global
            )

        _query_per_second_per_region = d.pop("query_per_second_per_region", UNSET)
        query_per_second_per_region: Union[Unset, QPS]
        if isinstance(_query_per_second_per_region, Unset):
            query_per_second_per_region = UNSET
        else:
            query_per_second_per_region = QPS.from_dict(_query_per_second_per_region)

        _query_per_second_per_region_per_code = d.pop(
            "query_per_second_per_region_per_code", UNSET
        )
        query_per_second_per_region_per_code: Union[
            Unset, ResourceEnvironmentMetricsQueryPerSecondPerRegionPerCode
        ]
        if isinstance(_query_per_second_per_region_per_code, Unset):
            query_per_second_per_region_per_code = UNSET
        else:
            query_per_second_per_region_per_code = (
                ResourceEnvironmentMetricsQueryPerSecondPerRegionPerCode.from_dict(
                    _query_per_second_per_region_per_code
                )
            )

        resource_environment_metrics = cls(
            inference_per_second_global=inference_per_second_global,
            inference_per_second_per_region=inference_per_second_per_region,
            query_per_second_global=query_per_second_global,
            query_per_second_per_code_global=query_per_second_per_code_global,
            query_per_second_per_region=query_per_second_per_region,
            query_per_second_per_region_per_code=query_per_second_per_region_per_code,
        )

        resource_environment_metrics.additional_properties = d
        return resource_environment_metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.pending_invitation_render_invited_by import PendingInvitationRenderInvitedBy
    from ..models.pending_invitation_render_workspace import PendingInvitationRenderWorkspace
    from ..models.pending_invitation_workspace_details import PendingInvitationWorkspaceDetails


T = TypeVar("T", bound="PendingInvitationRender")


@_attrs_define
class PendingInvitationRender:
    """Pending invitation in workspace

    Attributes:
        email (Union[Unset, str]): User email
        invited_at (Union[Unset, str]): Invitation date
        invited_by (Union[Unset, PendingInvitationRenderInvitedBy]): Invited by
        role (Union[Unset, str]): ACL role
        workspace (Union[Unset, PendingInvitationRenderWorkspace]): Workspace
        workspace_details (Union[Unset, PendingInvitationWorkspaceDetails]): Workspace details
    """

    email: Union[Unset, str] = UNSET
    invited_at: Union[Unset, str] = UNSET
    invited_by: Union[Unset, "PendingInvitationRenderInvitedBy"] = UNSET
    role: Union[Unset, str] = UNSET
    workspace: Union[Unset, "PendingInvitationRenderWorkspace"] = UNSET
    workspace_details: Union[Unset, "PendingInvitationWorkspaceDetails"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        email = self.email

        invited_at = self.invited_at

        invited_by: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.invited_by, Unset):
            invited_by = self.invited_by.to_dict()

        role = self.role

        workspace: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.workspace, Unset):
            workspace = self.workspace.to_dict()

        workspace_details: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.workspace_details, Unset):
            workspace_details = self.workspace_details.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if email is not UNSET:
            field_dict["email"] = email
        if invited_at is not UNSET:
            field_dict["invited_at"] = invited_at
        if invited_by is not UNSET:
            field_dict["invited_by"] = invited_by
        if role is not UNSET:
            field_dict["role"] = role
        if workspace is not UNSET:
            field_dict["workspace"] = workspace
        if workspace_details is not UNSET:
            field_dict["workspace_details"] = workspace_details

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.pending_invitation_render_invited_by import PendingInvitationRenderInvitedBy
        from ..models.pending_invitation_render_workspace import PendingInvitationRenderWorkspace
        from ..models.pending_invitation_workspace_details import PendingInvitationWorkspaceDetails

        if not src_dict:
            return None
        d = src_dict.copy()
        email = d.pop("email", UNSET)

        invited_at = d.pop("invited_at", UNSET)

        _invited_by = d.pop("invited_by", UNSET)
        invited_by: Union[Unset, PendingInvitationRenderInvitedBy]
        if isinstance(_invited_by, Unset):
            invited_by = UNSET
        else:
            invited_by = PendingInvitationRenderInvitedBy.from_dict(_invited_by)

        role = d.pop("role", UNSET)

        _workspace = d.pop("workspace", UNSET)
        workspace: Union[Unset, PendingInvitationRenderWorkspace]
        if isinstance(_workspace, Unset):
            workspace = UNSET
        else:
            workspace = PendingInvitationRenderWorkspace.from_dict(_workspace)

        _workspace_details = d.pop("workspace_details", UNSET)
        workspace_details: Union[Unset, PendingInvitationWorkspaceDetails]
        if isinstance(_workspace_details, Unset):
            workspace_details = UNSET
        else:
            workspace_details = PendingInvitationWorkspaceDetails.from_dict(_workspace_details)

        pending_invitation_render = cls(
            email=email,
            invited_at=invited_at,
            invited_by=invited_by,
            role=role,
            workspace=workspace,
            workspace_details=workspace_details,
        )

        pending_invitation_render.additional_properties = d
        return pending_invitation_render

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from .apikey import ApiKeyProvider
from .authentication import (
    PublicProvider,
    RunClientWithCredentials,
    get_authentication_headers,
    new_client,
    new_client_from_settings,
    new_client_with_credentials,
)
from .credentials import (
    Config,
    ContextConfig,
    Credentials,
    WorkspaceConfig,
    load_credentials,
    load_credentials_from_settings,
)
from .device_mode import (
    BearerToken,
    DeviceLogin,
    DeviceLoginFinalizeRequest,
    DeviceLoginFinalizeResponse,
    DeviceLoginResponse,
)

__all__ = (
    "ApiKeyProvider",
    "PublicProvider",
    "RunClientWithCredentials",
    "new_client_with_credentials",
    "new_client_from_settings",
    "new_client",
    "get_authentication_headers",
    "Config",
    "ContextConfig",
    "Credentials",
    "WorkspaceConfig",
    "load_credentials",
    "load_credentials_from_settings",
    "BearerToken",
    "DeviceLogin",
    "DeviceLoginFinalizeRequest",
    "DeviceLoginFinalizeResponse",
    "DeviceLoginResponse",
)



from typing import Any, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="DeploymentServerlessConfigType0")


@_attrs_define
class DeploymentServerlessConfigType0:
    """Configuration for a serverless deployment

    Attributes:
        last_pod_retention_period (Union[None, Unset, str]): The minimum amount of time that the last replica will
            remain active AFTER a scale-to-zero decision is made
        max_num_replicas (Union[None, Unset, int]): The maximum number of replicas for the deployment.
        metric (Union[None, Unset, str]): Metric watched to make scaling decisions. Can be "cpu" or "memory" or "rps" or
            "concurrency"
        min_num_replicas (Union[None, Unset, int]): The minimum number of replicas for the deployment. Can be 0 or 1 (in
            which case the deployment is always running in at least one location).
        scale_down_delay (Union[None, Unset, str]): The time window which must pass at reduced concurrency before a
            scale-down decision is applied. This can be useful, for example, to keep containers around for a configurable
            duration to avoid a cold start penalty if new requests come in.
        scale_up_minimum (Union[None, Unset, int]): The minimum number of replicas that will be created when the
            deployment scales up from zero.
        stable_window (Union[None, Unset, str]): The sliding time window over which metrics are averaged to provide the
            input for scaling decisions
        target (Union[None, Unset, str]): Target value for the watched metric
    """

    last_pod_retention_period: Union[None, Unset, str] = UNSET
    max_num_replicas: Union[None, Unset, int] = UNSET
    metric: Union[None, Unset, str] = UNSET
    min_num_replicas: Union[None, Unset, int] = UNSET
    scale_down_delay: Union[None, Unset, str] = UNSET
    scale_up_minimum: Union[None, Unset, int] = UNSET
    stable_window: Union[None, Unset, str] = UNSET
    target: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        last_pod_retention_period: Union[None, Unset, str]
        if isinstance(self.last_pod_retention_period, Unset):
            last_pod_retention_period = UNSET
        else:
            last_pod_retention_period = self.last_pod_retention_period

        max_num_replicas: Union[None, Unset, int]
        if isinstance(self.max_num_replicas, Unset):
            max_num_replicas = UNSET
        else:
            max_num_replicas = self.max_num_replicas

        metric: Union[None, Unset, str]
        if isinstance(self.metric, Unset):
            metric = UNSET
        else:
            metric = self.metric

        min_num_replicas: Union[None, Unset, int]
        if isinstance(self.min_num_replicas, Unset):
            min_num_replicas = UNSET
        else:
            min_num_replicas = self.min_num_replicas

        scale_down_delay: Union[None, Unset, str]
        if isinstance(self.scale_down_delay, Unset):
            scale_down_delay = UNSET
        else:
            scale_down_delay = self.scale_down_delay

        scale_up_minimum: Union[None, Unset, int]
        if isinstance(self.scale_up_minimum, Unset):
            scale_up_minimum = UNSET
        else:
            scale_up_minimum = self.scale_up_minimum

        stable_window: Union[None, Unset, str]
        if isinstance(self.stable_window, Unset):
            stable_window = UNSET
        else:
            stable_window = self.stable_window

        target: Union[None, Unset, str]
        if isinstance(self.target, Unset):
            target = UNSET
        else:
            target = self.target

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if last_pod_retention_period is not UNSET:
            field_dict["last_pod_retention_period"] = last_pod_retention_period
        if max_num_replicas is not UNSET:
            field_dict["max_num_replicas"] = max_num_replicas
        if metric is not UNSET:
            field_dict["metric"] = metric
        if min_num_replicas is not UNSET:
            field_dict["min_num_replicas"] = min_num_replicas
        if scale_down_delay is not UNSET:
            field_dict["scale_down_delay"] = scale_down_delay
        if scale_up_minimum is not UNSET:
            field_dict["scale_up_minimum"] = scale_up_minimum
        if stable_window is not UNSET:
            field_dict["stable_window"] = stable_window
        if target is not UNSET:
            field_dict["target"] = target

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_last_pod_retention_period(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        last_pod_retention_period = _parse_last_pod_retention_period(
            d.pop("last_pod_retention_period", UNSET)
        )

        def _parse_max_num_replicas(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        max_num_replicas = _parse_max_num_replicas(d.pop("max_num_replicas", UNSET))

        def _parse_metric(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        metric = _parse_metric(d.pop("metric", UNSET))

        def _parse_min_num_replicas(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        min_num_replicas = _parse_min_num_replicas(d.pop("min_num_replicas", UNSET))

        def _parse_scale_down_delay(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        scale_down_delay = _parse_scale_down_delay(d.pop("scale_down_delay", UNSET))

        def _parse_scale_up_minimum(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        scale_up_minimum = _parse_scale_up_minimum(d.pop("scale_up_minimum", UNSET))

        def _parse_stable_window(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        stable_window = _parse_stable_window(d.pop("stable_window", UNSET))

        def _parse_target(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        target = _parse_target(d.pop("target", UNSET))

        deployment_serverless_config_type_0 = cls(
            last_pod_retention_period=last_pod_retention_period,
            max_num_replicas=max_num_replicas,
            metric=metric,
            min_num_replicas=min_num_replicas,
            scale_down_delay=scale_down_delay,
            scale_up_minimum=scale_up_minimum,
            stable_window=stable_window,
            target=target,
        )

        deployment_serverless_config_type_0.additional_properties = d
        return deployment_serverless_config_type_0

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import os
from logging import getLogger
from typing import List, Tuple, Type, Union

from langchain_core.language_models.chat_models import BaseChatModel
from langgraph.graph.graph import CompiledGraph
from pydantic import Field
from pydantic_settings import (
    BaseSettings,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
    YamlConfigSettingsSource,
)

from beamlit.api.functions import get_function_deployment
from beamlit.api.models import get_model_deployment
from beamlit.client import AuthenticatedClient
from beamlit.common.logger import init as init_logger
from beamlit.models.agent_deployment import AgentDeployment
from beamlit.models.function_deployment import FunctionDeployment
from beamlit.models.model_deployment import ModelDeployment
from beamlit.types import UNSET, Unset

global SETTINGS
SETTINGS = None


def get_settings():
    return SETTINGS


class SettingsAgent(BaseSettings):
    agent: Union[None, CompiledGraph, BaseChatModel] = None
    chain: Union[Unset, List[AgentDeployment]] = UNSET
    model: Union[Unset, ModelDeployment] = UNSET
    functions: Union[Unset, List[FunctionDeployment]] = UNSET
    functions_directory: str = Field(default="src/functions")
    chat_model: Union[None, BaseChatModel] = None
    module: str = Field(default="main.main")


class SettingsAuthentication(BaseSettings):
    api_key: Union[None, str] = None
    jwt: Union[None, str] = None
    client_credentials: Union[None, str] = None


class SettingsServer(BaseSettings):
    module: str = Field(default="main.main")
    port: int = Field(default=80)
    host: str = Field(default="0.0.0.0")


class Settings(BaseSettings):
    model_config = SettingsConfigDict(
        yaml_file="beamlit.yaml",
        env_prefix="bl_",
        env_nested_delimiter="_",
    )

    workspace: str
    environment: str = Field(default="production")
    type: str = Field(default="agent")
    name: str = Field(default="beamlit-agent")
    base_url: str = Field(default="https://api.beamlit.dev/v0")
    run_url: str = Field(default="https://run.beamlit.dev")
    log_level: str = Field(default="INFO")
    agent: SettingsAgent = SettingsAgent()
    server: SettingsServer = SettingsServer()
    authentication: SettingsAuthentication = SettingsAuthentication()

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: Type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> Tuple[PydanticBaseSettingsSource, ...]:
        return (
            env_settings,
            dotenv_settings,
            file_secret_settings,
            YamlConfigSettingsSource(settings_cls),
            init_settings,
        )


def init_agent(
    client: AuthenticatedClient,
    destination: str = f"{os.getcwd()}/src/beamlit_generated.py",
):
    from beamlit.api.agents import get_agent_deployment
    from beamlit.common.generate import generate

    logger = getLogger(__name__)
    settings = get_settings()
    # Init configuration from environment variables
    if settings.agent.functions or settings.agent.chain:
        return

    # Init configuration from beamlit control plane
    name = settings.name
    env = settings.environment

    agent_deployment = get_agent_deployment.sync(name, env, client=client)
    function_deployments = []
    agent_chain_deployments = []
    if agent_deployment.functions:
        for function in agent_deployment.functions:
            function_deployment = get_function_deployment.sync(function, env, client=client)
            function_deployments.append(function_deployment)
        settings.agent.functions = function_deployments

    if agent_deployment.agent_chain:
        for chain in agent_deployment.agent_chain:
            if chain.enabled:
                agent_deployment = get_agent_deployment.sync(chain.name, env, client=client)
                if chain.description:
                    agent_deployment.description = chain.description
                agent_chain_deployments.append(agent_deployment)
        settings.agent.chain = agent_chain_deployments
    if agent_deployment.model:
        model_deployment = get_model_deployment.sync(agent_deployment.model, env, client=client)
        settings.agent.model = model_deployment

    content_generate = generate(destination, dry_run=True)
    compared_content = None
    if os.path.exists(destination):
        compared_content = open(destination).read()

    if not os.path.exists(destination) or (
        compared_content and content_generate != compared_content
    ):
        logger.info("Generating agent code")
        generate(destination)


def init() -> Settings:
    """Parse the beamlit.yaml file to get configurations."""
    from beamlit.authentication.credentials import current_context

    global SETTINGS

    context = current_context()
    kwargs = {}
    if context.workspace:
        kwargs["workspace"] = context.workspace
    if context.environment:
        kwargs["environment"] = context.environment

    SETTINGS = Settings(**kwargs)
    init_logger(SETTINGS.log_level)

    return SETTINGS



from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Country")


@_attrs_define
class Country:
    """Configuration

    Attributes:
        display_name (Union[Unset, str]): Country display name
        name (Union[Unset, str]): Country code
    """

    display_name: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        display_name = self.display_name

        name = self.name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if display_name is not UNSET:
            field_dict["display_name"] = display_name
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        display_name = d.pop("display_name", UNSET)

        name = d.pop("name", UNSET)

        country = cls(
            display_name=display_name,
            name=name,
        )

        country.additional_properties = d
        return country

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="RuntimeType0Resources")


@_attrs_define
class RuntimeType0Resources:
    """The resources for the deployment. Should be a Kubernetes ResourceRequirements type

    Attributes:
        requests (Union[Unset, str]):
    """

    requests: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        requests = self.requests

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if requests is not UNSET:
            field_dict["requests"] = requests

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        requests = d.pop("requests", UNSET)

        runtime_type_0_resources = cls(
            requests=requests,
        )

        runtime_type_0_resources.additional_properties = d
        return runtime_type_0_resources

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="EnvironmentSpec")


@_attrs_define
class EnvironmentSpec:
    """Environment specification

    Attributes:
        policies (Union[Unset, List[str]]):
    """

    policies: Union[Unset, List[str]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        policies: Union[Unset, List[str]] = UNSET
        if not isinstance(self.policies, Unset):
            policies = self.policies

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if policies is not UNSET:
            field_dict["policies"] = policies

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        policies = cast(List[str], d.pop("policies", UNSET))

        environment_spec = cls(
            policies=policies,
        )

        environment_spec.additional_properties = d
        return environment_spec

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ModelProviderRef")


@_attrs_define
class ModelProviderRef:
    """Reference to a model provider

    Attributes:
        kind (Union[Unset, str]): The kind of the model provider, can be Provider
        name (Union[Unset, str]): The name of the model provider
    """

    kind: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        kind = self.kind

        name = self.name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if kind is not UNSET:
            field_dict["kind"] = kind
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        kind = d.pop("kind", UNSET)

        name = d.pop("name", UNSET)

        model_provider_ref = cls(
            kind=kind,
            name=name,
        )

        model_provider_ref.additional_properties = d
        return model_provider_ref

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.integration_config import IntegrationConfig
    from ..models.integration_secret import IntegrationSecret
    from ..models.labels_type_0 import LabelsType0


T = TypeVar("T", bound="Integration")


@_attrs_define
class Integration:
    """Model provider

    Attributes:
        created_at (Union[Unset, str]): The date and time when the resource was created
        created_by (Union[Unset, str]): The user or service account who created the resource
        updated_at (Union[Unset, str]): The date and time when the resource was updated
        updated_by (Union[Unset, str]): The user or service account who updated the resource
        config (Union[Unset, IntegrationConfig]): Integration config
        display_name (Union[Unset, str]): Integration display name
        labels (Union['LabelsType0', None, Unset]): Labels
        name (Union[Unset, str]): Integration name
        secret (Union[Unset, IntegrationSecret]): Integration secret
        type_ (Union[Unset, str]): Integration type
        workspace (Union[Unset, str]): Workspace name
    """

    created_at: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET
    updated_at: Union[Unset, str] = UNSET
    updated_by: Union[Unset, str] = UNSET
    config: Union[Unset, "IntegrationConfig"] = UNSET
    display_name: Union[Unset, str] = UNSET
    labels: Union["LabelsType0", None, Unset] = UNSET
    name: Union[Unset, str] = UNSET
    secret: Union[Unset, "IntegrationSecret"] = UNSET
    type_: Union[Unset, str] = UNSET
    workspace: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.labels_type_0 import LabelsType0

        created_at = self.created_at

        created_by = self.created_by

        updated_at = self.updated_at

        updated_by = self.updated_by

        config: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.config, Unset):
            config = self.config.to_dict()

        display_name = self.display_name

        labels: Union[None, Unset, dict[str, Any]]
        if isinstance(self.labels, Unset):
            labels = UNSET
        elif isinstance(self.labels, LabelsType0):
            labels = self.labels.to_dict()
        else:
            labels = self.labels

        name = self.name

        secret: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.secret, Unset):
            secret = self.secret.to_dict()

        type_ = self.type_

        workspace = self.workspace

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if created_by is not UNSET:
            field_dict["created_by"] = created_by
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if updated_by is not UNSET:
            field_dict["updated_by"] = updated_by
        if config is not UNSET:
            field_dict["config"] = config
        if display_name is not UNSET:
            field_dict["display_name"] = display_name
        if labels is not UNSET:
            field_dict["labels"] = labels
        if name is not UNSET:
            field_dict["name"] = name
        if secret is not UNSET:
            field_dict["secret"] = secret
        if type_ is not UNSET:
            field_dict["type"] = type_
        if workspace is not UNSET:
            field_dict["workspace"] = workspace

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.integration_config import IntegrationConfig
        from ..models.integration_secret import IntegrationSecret
        from ..models.labels_type_0 import LabelsType0

        if not src_dict:
            return None
        d = src_dict.copy()
        created_at = d.pop("created_at", UNSET)

        created_by = d.pop("created_by", UNSET)

        updated_at = d.pop("updated_at", UNSET)

        updated_by = d.pop("updated_by", UNSET)

        _config = d.pop("config", UNSET)
        config: Union[Unset, IntegrationConfig]
        if isinstance(_config, Unset):
            config = UNSET
        else:
            config = IntegrationConfig.from_dict(_config)

        display_name = d.pop("display_name", UNSET)

        def _parse_labels(data: object) -> Union["LabelsType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_labels_type_0 = LabelsType0.from_dict(data)

                return componentsschemas_labels_type_0
            except:  # noqa: E722
                pass
            return cast(Union["LabelsType0", None, Unset], data)

        labels = _parse_labels(d.pop("labels", UNSET))

        name = d.pop("name", UNSET)

        _secret = d.pop("secret", UNSET)
        secret: Union[Unset, IntegrationSecret]
        if isinstance(_secret, Unset):
            secret = UNSET
        else:
            secret = IntegrationSecret.from_dict(_secret)

        type_ = d.pop("type", UNSET)

        workspace = d.pop("workspace", UNSET)

        integration = cls(
            created_at=created_at,
            created_by=created_by,
            updated_at=updated_at,
            updated_by=updated_by,
            config=config,
            display_name=display_name,
            labels=labels,
            name=name,
            secret=secret,
            type_=type_,
            workspace=workspace,
        )

        integration.additional_properties = d
        return integration

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="IntegrationConnectionSecret")


@_attrs_define
class IntegrationConnectionSecret:
    """Integration secret

    Attributes:
        api_key (Union[Unset, str]): The API key to use for the integration
    """

    api_key: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        api_key = self.api_key

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if api_key is not UNSET:
            field_dict["api_key"] = api_key

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        api_key = d.pop("api_key", UNSET)

        integration_connection_secret = cls(
            api_key=api_key,
        )

        integration_connection_secret.additional_properties = d
        return integration_connection_secret

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .acl import ACL
from .agent import Agent
from .agent_chain import AgentChain
from .agent_configuration import AgentConfiguration
from .agent_deployment import AgentDeployment
from .agent_deployment_configuration import AgentDeploymentConfiguration
from .agent_deployment_history import AgentDeploymentHistory
from .agent_deployment_history_event import AgentDeploymentHistoryEvent
from .agent_deployment_pod_template import AgentDeploymentPodTemplate
from .agent_release import AgentRelease
from .agent_with_deployments import AgentWithDeployments
from .api_key import ApiKey
from .authentication_provider_model import AuthenticationProviderModel
from .authentication_provider_organization import AuthenticationProviderOrganization
from .configuration import Configuration
from .continent import Continent
from .country import Country
from .create_api_key_for_service_account_body import CreateApiKeyForServiceAccountBody
from .create_workspace_service_account_body import CreateWorkspaceServiceAccountBody
from .create_workspace_service_account_response_200 import CreateWorkspaceServiceAccountResponse200
from .delete_workspace_service_account_response_200 import DeleteWorkspaceServiceAccountResponse200
from .deployment_configuration import DeploymentConfiguration
from .deployment_configurations import DeploymentConfigurations
from .deployment_serverless_config import DeploymentServerlessConfig
from .environment import Environment
from .environment_metrics import EnvironmentMetrics
from .flavor import Flavor
from .function import Function
from .function_configuration import FunctionConfiguration
from .function_deployment import FunctionDeployment
from .function_deployment_configuration import FunctionDeploymentConfiguration
from .function_deployment_pod_template import FunctionDeploymentPodTemplate
from .function_kit import FunctionKit
from .function_provider_ref import FunctionProviderRef
from .function_release import FunctionRelease
from .function_with_deployments import FunctionWithDeployments
from .get_workspace_service_accounts_response_200_item import GetWorkspaceServiceAccountsResponse200Item
from .increase_and_rate_metric import IncreaseAndRateMetric
from .integration import Integration
from .integration_config import IntegrationConfig
from .integration_connection import IntegrationConnection
from .integration_connection_config import IntegrationConnectionConfig
from .integration_connection_secret import IntegrationConnectionSecret
from .integration_model import IntegrationModel
from .integration_secret import IntegrationSecret
from .invite_workspace_user_body import InviteWorkspaceUserBody
from .labels_type_0 import LabelsType0
from .location import Location
from .location_response import LocationResponse
from .metric import Metric
from .metrics import Metrics
from .model import Model
from .model_deployment import ModelDeployment
from .model_deployment_log import ModelDeploymentLog
from .model_deployment_metrics import ModelDeploymentMetrics
from .model_deployment_metrics_inference_per_second_per_region import ModelDeploymentMetricsInferencePerSecondPerRegion
from .model_deployment_metrics_query_per_second_per_region_per_code import (
    ModelDeploymentMetricsQueryPerSecondPerRegionPerCode,
)
from .model_deployment_pod_template import ModelDeploymentPodTemplate
from .model_metrics import ModelMetrics
from .model_provider import ModelProvider
from .model_provider_ref import ModelProviderRef
from .model_release import ModelRelease
from .model_with_deployments import ModelWithDeployments
from .pending_invitation import PendingInvitation
from .pending_invitation_accept import PendingInvitationAccept
from .pending_invitation_render import PendingInvitationRender
from .pending_invitation_render_invited_by import PendingInvitationRenderInvitedBy
from .pending_invitation_render_workspace import PendingInvitationRenderWorkspace
from .pending_invitation_workspace_details import PendingInvitationWorkspaceDetails
from .policy import Policy
from .policy_location import PolicyLocation
from .provider_config import ProviderConfig
from .qps import QPS
from .resource_deployment_log import ResourceDeploymentLog
from .resource_deployment_metrics import ResourceDeploymentMetrics
from .resource_deployment_metrics_inference_per_region import ResourceDeploymentMetricsInferencePerRegion
from .resource_deployment_metrics_inference_per_second_per_region import (
    ResourceDeploymentMetricsInferencePerSecondPerRegion,
)
from .resource_deployment_metrics_query_per_region_per_code import ResourceDeploymentMetricsQueryPerRegionPerCode
from .resource_deployment_metrics_query_per_second_per_region_per_code import (
    ResourceDeploymentMetricsQueryPerSecondPerRegionPerCode,
)
from .resource_metrics import ResourceMetrics
from .runtime import Runtime
from .runtime_readiness_probe import RuntimeReadinessProbe
from .runtime_resources import RuntimeResources
from .serverless_config import ServerlessConfig
from .standard_fields_dynamo_db import StandardFieldsDynamoDb
from .store_agent import StoreAgent
from .store_agent_configuration import StoreAgentConfiguration
from .store_agent_labels import StoreAgentLabels
from .store_configuration import StoreConfiguration
from .store_configuration_option import StoreConfigurationOption
from .store_function import StoreFunction
from .store_function_configuration import StoreFunctionConfiguration
from .store_function_kit import StoreFunctionKit
from .store_function_labels import StoreFunctionLabels
from .store_function_parameter import StoreFunctionParameter
from .update_workspace_service_account_body import UpdateWorkspaceServiceAccountBody
from .update_workspace_service_account_response_200 import UpdateWorkspaceServiceAccountResponse200
from .update_workspace_user_role_body import UpdateWorkspaceUserRoleBody
from .workspace import Workspace
from .workspace_labels import WorkspaceLabels
from .workspace_user import WorkspaceUser

__all__ = (
    "ACL",
    "Agent",
    "AgentChain",
    "AgentConfiguration",
    "AgentDeployment",
    "AgentDeploymentConfiguration",
    "AgentDeploymentHistory",
    "AgentDeploymentHistoryEvent",
    "AgentDeploymentPodTemplate",
    "AgentRelease",
    "AgentWithDeployments",
    "ApiKey",
    "AuthenticationProviderModel",
    "AuthenticationProviderOrganization",
    "Configuration",
    "Continent",
    "Country",
    "CreateApiKeyForServiceAccountBody",
    "CreateWorkspaceServiceAccountBody",
    "CreateWorkspaceServiceAccountResponse200",
    "DeleteWorkspaceServiceAccountResponse200",
    "DeploymentConfiguration",
    "DeploymentConfigurations",
    "DeploymentServerlessConfig",
    "Environment",
    "EnvironmentMetrics",
    "Flavor",
    "Function",
    "FunctionConfiguration",
    "FunctionDeployment",
    "FunctionDeploymentConfiguration",
    "FunctionDeploymentPodTemplate",
    "FunctionKit",
    "FunctionProviderRef",
    "FunctionRelease",
    "FunctionWithDeployments",
    "GetWorkspaceServiceAccountsResponse200Item",
    "IncreaseAndRateMetric",
    "Integration",
    "IntegrationConfig",
    "IntegrationConnection",
    "IntegrationConnectionConfig",
    "IntegrationConnectionSecret",
    "IntegrationModel",
    "IntegrationSecret",
    "InviteWorkspaceUserBody",
    "LabelsType0",
    "Location",
    "LocationResponse",
    "Metric",
    "Metrics",
    "Model",
    "ModelDeployment",
    "ModelDeploymentLog",
    "ModelDeploymentMetrics",
    "ModelDeploymentMetricsInferencePerSecondPerRegion",
    "ModelDeploymentMetricsQueryPerSecondPerRegionPerCode",
    "ModelDeploymentPodTemplate",
    "ModelMetrics",
    "ModelProvider",
    "ModelProviderRef",
    "ModelRelease",
    "ModelWithDeployments",
    "PendingInvitation",
    "PendingInvitationAccept",
    "PendingInvitationRender",
    "PendingInvitationRenderInvitedBy",
    "PendingInvitationRenderWorkspace",
    "PendingInvitationWorkspaceDetails",
    "Policy",
    "PolicyLocation",
    "ProviderConfig",
    "QPS",
    "ResourceDeploymentLog",
    "ResourceDeploymentMetrics",
    "ResourceDeploymentMetricsInferencePerRegion",
    "ResourceDeploymentMetricsInferencePerSecondPerRegion",
    "ResourceDeploymentMetricsQueryPerRegionPerCode",
    "ResourceDeploymentMetricsQueryPerSecondPerRegionPerCode",
    "ResourceMetrics",
    "Runtime",
    "RuntimeReadinessProbe",
    "RuntimeResources",
    "ServerlessConfig",
    "StandardFieldsDynamoDb",
    "StoreAgent",
    "StoreAgentConfiguration",
    "StoreAgentLabels",
    "StoreConfiguration",
    "StoreConfigurationOption",
    "StoreFunction",
    "StoreFunctionConfiguration",
    "StoreFunctionKit",
    "StoreFunctionLabels",
    "StoreFunctionParameter",
    "UpdateWorkspaceServiceAccountBody",
    "UpdateWorkspaceServiceAccountResponse200",
    "UpdateWorkspaceUserRoleBody",
    "Workspace",
    "WorkspaceLabels",
    "WorkspaceUser",
)



# Import necessary modules
import ast
import importlib
import os
from logging import getLogger

from langgraph.checkpoint.memory import MemorySaver
from langgraph.prebuilt import create_react_agent

from beamlit.api.models import get_model_deployment
from beamlit.authentication import new_client
from beamlit.common.settings import get_settings, init
from beamlit.errors import UnexpectedStatus
from beamlit.models import AgentDeployment

from .chat import get_chat_model


def get_functions(dir="src/functions", from_decorator="function"):
    functions = []
    logger = getLogger(__name__)

    # Walk through all Python files in functions directory and subdirectories
    for root, _, files in os.walk(dir):
        for file in files:
            if file.endswith(".py"):
                file_path = os.path.join(root, file)
                # Read and compile the file content
                with open(file_path) as f:
                    try:
                        file_content = f.read()
                        # Parse the file content to find decorated functions
                        tree = ast.parse(file_content)

                        # Look for function definitions with decorators
                        for node in ast.walk(tree):
                            if (
                                not isinstance(node, ast.FunctionDef)
                                or len(node.decorator_list) == 0
                            ):
                                continue
                            decorator = node.decorator_list[0]

                            decorator_name = ""
                            if isinstance(decorator, ast.Call):
                                decorator_name = decorator.func.id
                            if isinstance(decorator, ast.Name):
                                decorator_name = decorator.id

                            if decorator_name == from_decorator:
                                # Get the function name and decorator name
                                func_name = node.name

                                # Import the module to get the actual function
                                spec = importlib.util.spec_from_file_location(func_name, file_path)
                                module = importlib.util.module_from_spec(spec)
                                spec.loader.exec_module(module)
                                # Check if kit=True in the decorator arguments
                                is_kit = False
                                if isinstance(decorator, ast.Call):
                                    for keyword in decorator.keywords:
                                        if keyword.arg == "kit" and isinstance(
                                            keyword.value, ast.Constant
                                        ):
                                            is_kit = keyword.value.value
                                if is_kit:
                                    kit_functions = get_functions(
                                        dir=os.path.join(root),
                                        from_decorator="kit",
                                    )
                                    functions.extend(kit_functions)

                                # Get the decorated function
                                if not is_kit and hasattr(module, func_name):
                                    func = getattr(module, func_name)
                                    functions.append(func)
                    except Exception as e:
                        logger.warning(f"Error processing {file_path}: {e!s}")
    return functions


def agent(
    bl_agent: AgentDeployment = None,
    chat_model=None,
    agent=None,
):
    settings = init()
    logger = getLogger(__name__)

    def wrapper(func):
        settings = get_settings()

        def wrapped(*args, **kwargs):
            return func(
                settings.agent.agent,
                settings.agent.chat_model,
                settings.agent.functions,
                *args,
                **kwargs,
            )

        return wrapped

    # Initialize functions array to store decorated functions
    functions = get_functions(dir=settings.agent.functions_directory)
    settings.agent.functions = functions

    if bl_agent.model and chat_model is None:
        client = new_client()
        try:
            response = get_model_deployment.sync_detailed(
                bl_agent.model, settings.environment, client=client
            )
            settings.agent.model = response.parsed
        except UnexpectedStatus as e:
            if e.status_code == 404 and settings.environment != "production":
                try:
                    response = get_model_deployment.sync_detailed(
                        bl_agent.model, "production", client=client
                    )
                    settings.agent.model = response.parsed
                except UnexpectedStatus as e:
                    if e.status_code == 404:
                        raise ValueError(f"Model {bl_agent.model} not found")
            else:
                raise e
        chat_model = get_chat_model(settings.agent.model)
        settings.agent.chat_model = chat_model
        runtime = settings.agent.model.runtime
        logger.info(f"Chat model configured, using: {runtime.type_}:{runtime.model}")

    if len(functions) == 0:
        raise ValueError(
            "You must define at least one function, you can define this function in directory "
            f'"{settings.agent.functions_directory}". Here is a sample function you can use:\n\n'
            "from beamlit.functions import function\n\n"
            "@function()\n"
            "def hello_world(query: str):\n"
            "    return 'Hello, world!'\n"
        )

    if agent is None and chat_model is not None:
        memory = MemorySaver()
        agent = create_react_agent(chat_model, functions, checkpointer=memory)
        settings.agent.agent = agent

    return wrapper



from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.deployment_serverless_config import DeploymentServerlessConfig
    from ..models.flavor import Flavor
    from ..models.function_deployment_configuration import FunctionDeploymentConfiguration
    from ..models.function_deployment_pod_template import FunctionDeploymentPodTemplate
    from ..models.function_kit import FunctionKit
    from ..models.labels_type_0 import LabelsType0
    from ..models.runtime import Runtime
    from ..models.store_function_parameter import StoreFunctionParameter


T = TypeVar("T", bound="FunctionDeployment")


@_attrs_define
class FunctionDeployment:
    """Function deployment configuration

    Attributes:
        created_at (Union[Unset, str]): The date and time when the resource was created
        created_by (Union[Unset, str]): The user or service account who created the resource
        updated_at (Union[Unset, str]): The date and time when the resource was updated
        updated_by (Union[Unset, str]): The user or service account who updated the resource
        configuration (Union[Unset, FunctionDeploymentConfiguration]): Function configuration, this is a key value
            storage. In your function you can retrieve the value with config[key]
        description (Union[Unset, str]): Function description, very important for the agent function to work with an LLM
        enabled (Union[Unset, bool]): Whether the function deployment is enabled
        environment (Union[Unset, str]): The name of the environment
        flavors (Union[Unset, list['Flavor']]): Types of hardware available for deployments
        function (Union[Unset, str]): The name of the function
        integration_connections (Union[Unset, list[str]]):
        kit (Union[Unset, list['FunctionKit']]): The kit of the function deployment
        labels (Union['LabelsType0', None, Unset]): Labels
        parameters (Union[Unset, list['StoreFunctionParameter']]): Function parameters, for your function to be callable
            with Agent
        pod_template (Union[Unset, FunctionDeploymentPodTemplate]): The pod template, should be a valid Kubernetes pod
            template
        policies (Union[Unset, list[str]]):
        runtime (Union[Unset, Runtime]): Set of configurations for a deployment
        serverless_config (Union[Unset, DeploymentServerlessConfig]): Configuration for a serverless deployment
        store_id (Union[Unset, str]): Create from a store registered function
        workspace (Union[Unset, str]): The workspace the function deployment belongs to
    """

    created_at: Union[Unset, str] = UNSET
    created_by: Union[Unset, str] = UNSET
    updated_at: Union[Unset, str] = UNSET
    updated_by: Union[Unset, str] = UNSET
    configuration: Union[Unset, "FunctionDeploymentConfiguration"] = UNSET
    description: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    environment: Union[Unset, str] = UNSET
    flavors: Union[Unset, list["Flavor"]] = UNSET
    function: Union[Unset, str] = UNSET
    integration_connections: Union[Unset, list[str]] = UNSET
    kit: Union[Unset, list["FunctionKit"]] = UNSET
    labels: Union["LabelsType0", None, Unset] = UNSET
    parameters: Union[Unset, list["StoreFunctionParameter"]] = UNSET
    pod_template: Union[Unset, "FunctionDeploymentPodTemplate"] = UNSET
    policies: Union[Unset, list[str]] = UNSET
    runtime: Union[Unset, "Runtime"] = UNSET
    serverless_config: Union[Unset, "DeploymentServerlessConfig"] = UNSET
    store_id: Union[Unset, str] = UNSET
    workspace: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.labels_type_0 import LabelsType0

        created_at = self.created_at

        created_by = self.created_by

        updated_at = self.updated_at

        updated_by = self.updated_by

        configuration: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.configuration, Unset):
            configuration = self.configuration.to_dict()

        description = self.description

        enabled = self.enabled

        environment = self.environment

        flavors: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.flavors, Unset):
            flavors = []
            for componentsschemas_flavors_item_data in self.flavors:
                componentsschemas_flavors_item = componentsschemas_flavors_item_data.to_dict()
                flavors.append(componentsschemas_flavors_item)

        function = self.function

        integration_connections: Union[Unset, list[str]] = UNSET
        if not isinstance(self.integration_connections, Unset):
            integration_connections = self.integration_connections

        kit: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.kit, Unset):
            kit = []
            for kit_item_data in self.kit:
                kit_item = kit_item_data.to_dict()
                kit.append(kit_item)

        labels: Union[None, Unset, dict[str, Any]]
        if isinstance(self.labels, Unset):
            labels = UNSET
        elif isinstance(self.labels, LabelsType0):
            labels = self.labels.to_dict()
        else:
            labels = self.labels

        parameters: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.parameters, Unset):
            parameters = []
            for parameters_item_data in self.parameters:
                parameters_item = parameters_item_data.to_dict()
                parameters.append(parameters_item)

        pod_template: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.pod_template, Unset):
            pod_template = self.pod_template.to_dict()

        policies: Union[Unset, list[str]] = UNSET
        if not isinstance(self.policies, Unset):
            policies = self.policies

        runtime: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.runtime, Unset):
            runtime = self.runtime.to_dict()

        serverless_config: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.serverless_config, Unset):
            serverless_config = self.serverless_config.to_dict()

        store_id = self.store_id

        workspace = self.workspace

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if created_by is not UNSET:
            field_dict["created_by"] = created_by
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if updated_by is not UNSET:
            field_dict["updated_by"] = updated_by
        if configuration is not UNSET:
            field_dict["configuration"] = configuration
        if description is not UNSET:
            field_dict["description"] = description
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if environment is not UNSET:
            field_dict["environment"] = environment
        if flavors is not UNSET:
            field_dict["flavors"] = flavors
        if function is not UNSET:
            field_dict["function"] = function
        if integration_connections is not UNSET:
            field_dict["integration_connections"] = integration_connections
        if kit is not UNSET:
            field_dict["kit"] = kit
        if labels is not UNSET:
            field_dict["labels"] = labels
        if parameters is not UNSET:
            field_dict["parameters"] = parameters
        if pod_template is not UNSET:
            field_dict["pod_template"] = pod_template
        if policies is not UNSET:
            field_dict["policies"] = policies
        if runtime is not UNSET:
            field_dict["runtime"] = runtime
        if serverless_config is not UNSET:
            field_dict["serverless_config"] = serverless_config
        if store_id is not UNSET:
            field_dict["store_id"] = store_id
        if workspace is not UNSET:
            field_dict["workspace"] = workspace

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.deployment_serverless_config import DeploymentServerlessConfig
        from ..models.flavor import Flavor
        from ..models.function_deployment_configuration import FunctionDeploymentConfiguration
        from ..models.function_deployment_pod_template import FunctionDeploymentPodTemplate
        from ..models.function_kit import FunctionKit
        from ..models.labels_type_0 import LabelsType0
        from ..models.runtime import Runtime
        from ..models.store_function_parameter import StoreFunctionParameter

        if not src_dict:
            return None
        d = src_dict.copy()
        created_at = d.pop("created_at", UNSET)

        created_by = d.pop("created_by", UNSET)

        updated_at = d.pop("updated_at", UNSET)

        updated_by = d.pop("updated_by", UNSET)

        _configuration = d.pop("configuration", UNSET)
        configuration: Union[Unset, FunctionDeploymentConfiguration]
        if isinstance(_configuration, Unset):
            configuration = UNSET
        else:
            configuration = FunctionDeploymentConfiguration.from_dict(_configuration)

        description = d.pop("description", UNSET)

        enabled = d.pop("enabled", UNSET)

        environment = d.pop("environment", UNSET)

        flavors = []
        _flavors = d.pop("flavors", UNSET)
        for componentsschemas_flavors_item_data in _flavors or []:
            componentsschemas_flavors_item = Flavor.from_dict(componentsschemas_flavors_item_data)

            flavors.append(componentsschemas_flavors_item)

        function = d.pop("function", UNSET)

        integration_connections = cast(list[str], d.pop("integration_connections", UNSET))

        kit = []
        _kit = d.pop("kit", UNSET)
        for kit_item_data in _kit or []:
            kit_item = FunctionKit.from_dict(kit_item_data)

            kit.append(kit_item)

        def _parse_labels(data: object) -> Union["LabelsType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_labels_type_0 = LabelsType0.from_dict(data)

                return componentsschemas_labels_type_0
            except:  # noqa: E722
                pass
            return cast(Union["LabelsType0", None, Unset], data)

        labels = _parse_labels(d.pop("labels", UNSET))

        parameters = []
        _parameters = d.pop("parameters", UNSET)
        for parameters_item_data in _parameters or []:
            parameters_item = StoreFunctionParameter.from_dict(parameters_item_data)

            parameters.append(parameters_item)

        _pod_template = d.pop("pod_template", UNSET)
        pod_template: Union[Unset, FunctionDeploymentPodTemplate]
        if isinstance(_pod_template, Unset):
            pod_template = UNSET
        else:
            pod_template = FunctionDeploymentPodTemplate.from_dict(_pod_template)

        policies = cast(list[str], d.pop("policies", UNSET))

        _runtime = d.pop("runtime", UNSET)
        runtime: Union[Unset, Runtime]
        if isinstance(_runtime, Unset):
            runtime = UNSET
        else:
            runtime = Runtime.from_dict(_runtime)

        _serverless_config = d.pop("serverless_config", UNSET)
        serverless_config: Union[Unset, DeploymentServerlessConfig]
        if isinstance(_serverless_config, Unset):
            serverless_config = UNSET
        else:
            serverless_config = DeploymentServerlessConfig.from_dict(_serverless_config)

        store_id = d.pop("store_id", UNSET)

        workspace = d.pop("workspace", UNSET)

        function_deployment = cls(
            created_at=created_at,
            created_by=created_by,
            updated_at=updated_at,
            updated_by=updated_by,
            configuration=configuration,
            description=description,
            enabled=enabled,
            environment=environment,
            flavors=flavors,
            function=function,
            integration_connections=integration_connections,
            kit=kit,
            labels=labels,
            parameters=parameters,
            pod_template=pod_template,
            policies=policies,
            runtime=runtime,
            serverless_config=serverless_config,
            store_id=store_id,
            workspace=workspace,
        )

        function_deployment.additional_properties = d
        return function_deployment

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.workspace import Workspace


T = TypeVar("T", bound="PendingInvitationAccept")


@_attrs_define
class PendingInvitationAccept:
    """Pending invitation accept

    Attributes:
        email (Union[Unset, str]): User email
        workspace (Union[Unset, Workspace]): Workspace
    """

    email: Union[Unset, str] = UNSET
    workspace: Union[Unset, "Workspace"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        email = self.email

        workspace: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.workspace, Unset):
            workspace = self.workspace.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if email is not UNSET:
            field_dict["email"] = email
        if workspace is not UNSET:
            field_dict["workspace"] = workspace

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.workspace import Workspace

        if not src_dict:
            return None
        d = src_dict.copy()
        email = d.pop("email", UNSET)

        _workspace = d.pop("workspace", UNSET)
        workspace: Union[Unset, Workspace]
        if isinstance(_workspace, Unset):
            workspace = UNSET
        else:
            workspace = Workspace.from_dict(_workspace)

        pending_invitation_accept = cls(
            email=email,
            workspace=workspace,
        )

        pending_invitation_accept.additional_properties = d
        return pending_invitation_accept

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from logging import getLogger

from beamlit.authentication import get_authentication_headers, new_client
from beamlit.common.settings import get_settings
from beamlit.models import AgentDeployment

logger = getLogger(__name__)


def get_base_url(agent_model: AgentDeployment):
    settings = get_settings()
    return f"{settings.run_url}/{settings.workspace}/models/{agent_model.model}/v1"


def get_mistral_chat_model(**kwargs):
    from langchain_mistralai.chat_models import ChatMistralAI  # type: ignore

    return ChatMistralAI(**kwargs)


def get_openai_chat_model(**kwargs):
    from langchain_openai import ChatOpenAI  # type: ignore

    return ChatOpenAI(**kwargs)


def get_anthropic_chat_model(**kwargs):
    from langchain_anthropic import ChatAnthropic  # type: ignore

    return ChatAnthropic(**kwargs)


def get_chat_model(agent_model: AgentDeployment):
    settings = get_settings()
    client = new_client()

    headers = get_authentication_headers(settings)
    headers["X-Beamlit-Environment"] = agent_model.environment

    jwt = headers.pop("X-Beamlit-Authorization").replace("Bearer ", "")
    params = {"environment": agent_model.environment}
    chat_classes = {
        "openai": {
            "func": get_openai_chat_model,
            "kwargs": {
                "http_client": client.get_httpx_client(),
            },
        },
        "anthropic": {
            "func": get_anthropic_chat_model,
            "kwargs": {},
        },
        "mistral": {
            "func": get_mistral_chat_model,
            "kwargs": {
                "api_key": jwt,
            },
        },
    }

    if agent_model is None:
        raise ValueError("agent_model not found in configuration")
    if agent_model.runtime is None:
        raise ValueError("runtime not found in agent model")
    if agent_model.runtime.type_ is None:
        raise ValueError("type not found in runtime")
    if agent_model.runtime.model is None:
        raise ValueError("model not found in runtime")

    provider = agent_model.runtime.type_
    model = agent_model.runtime.model

    kwargs = {
        "model": model,
        "base_url": get_base_url(agent_model),
        "default_query": params,
        "default_headers": headers,
        "api_key": "fake_api_key",
        "temperature": 0,
    }
    chat_class = chat_classes.get(provider)
    if not chat_class:
        logger.warning(f"Provider {provider} not currently supported, defaulting to OpenAI")
        chat_class = chat_classes["openai"]
    if "kwargs" in chat_class:
        kwargs.update(chat_class["kwargs"])
    return chat_class["func"](**kwargs)



import asyncio
import uuid

from beamlit.agents import agent
from beamlit.models import AgentDeployment


@agent(
    AgentDeployment(
        agent="agent-custom",
        description="A chat agent using Beamlit to handle your tasks.",
        model="gpt-4o-mini",
    ),
)
async def main(agent, chat_model, tools, body, headers=None, query_params=None, **_):
    agent_config = {"configurable": {"thread_id": str(uuid.uuid4())}}
    if body.get("inputs"):
        body["input"] = body["inputs"]

    agent_body = {"messages": [("user", body["input"])]}
    responses = []

    for chunk in agent.stream(agent_body, config=agent_config):
        responses.append(chunk)
    content = responses[-1]
    return content["agent"]["messages"][-1].content


if __name__ == "__main__":

    async def check():
        response = await main(
            {"input": "What does 4+2 ?"},
            headers={"X-Beamlit-Sub": "123"},
            query_params={"debug": "false"},
        )
        print(response)

    asyncio.run(check())



from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.metric import Metric
    from ..models.qps import QPS
    from ..models.resource_deployment_metrics_inference_per_region import ResourceDeploymentMetricsInferencePerRegion
    from ..models.resource_deployment_metrics_inference_per_second_per_region import (
        ResourceDeploymentMetricsInferencePerSecondPerRegion,
    )
    from ..models.resource_deployment_metrics_query_per_region_per_code import (
        ResourceDeploymentMetricsQueryPerRegionPerCode,
    )
    from ..models.resource_deployment_metrics_query_per_second_per_region_per_code import (
        ResourceDeploymentMetricsQueryPerSecondPerRegionPerCode,
    )


T = TypeVar("T", bound="ResourceDeploymentMetrics")


@_attrs_define
class ResourceDeploymentMetrics:
    """Metrics for a single resource deployment (eg. model deployment, function deployment)

    Attributes:
        inference_global (Union[Unset, list['Metric']]): Array of metrics
        inference_per_region (Union[Unset, ResourceDeploymentMetricsInferencePerRegion]): Historical requests (in last
            24 hours) per location, for the model deployment
        inference_per_second_global (Union[Unset, list['Metric']]): Array of metrics
        inference_per_second_per_region (Union[Unset, ResourceDeploymentMetricsInferencePerSecondPerRegion]): Historical
            requests per second (RPS) per location, for the model deployment
        query_global (Union[Unset, float]): Number of requests done on the resource for the model deployment
        query_per_code_global (Union[Unset, QPS]): Query per second per element, can be per response status code (e.g.
            200, 400) or per location
        query_per_region (Union[Unset, QPS]): Query per second per element, can be per response status code (e.g. 200,
            400) or per location
        query_per_region_per_code (Union[Unset, ResourceDeploymentMetricsQueryPerRegionPerCode]): Number of requests
            done on the resource for the model deployment
        query_per_second_global (Union[Unset, float]): RPS value (in last 24 hours) for the model deployment globally
        query_per_second_per_code_global (Union[Unset, QPS]): Query per second per element, can be per response status
            code (e.g. 200, 400) or per location
        query_per_second_per_region (Union[Unset, QPS]): Query per second per element, can be per response status code
            (e.g. 200, 400) or per location
        query_per_second_per_region_per_code (Union[Unset, ResourceDeploymentMetricsQueryPerSecondPerRegionPerCode]):
            RPS value (in last 24 hours) per response status code per location, for the model deployment
    """

    inference_global: Union[Unset, list["Metric"]] = UNSET
    inference_per_region: Union[Unset, "ResourceDeploymentMetricsInferencePerRegion"] = UNSET
    inference_per_second_global: Union[Unset, list["Metric"]] = UNSET
    inference_per_second_per_region: Union[Unset, "ResourceDeploymentMetricsInferencePerSecondPerRegion"] = UNSET
    query_global: Union[Unset, float] = UNSET
    query_per_code_global: Union[Unset, "QPS"] = UNSET
    query_per_region: Union[Unset, "QPS"] = UNSET
    query_per_region_per_code: Union[Unset, "ResourceDeploymentMetricsQueryPerRegionPerCode"] = UNSET
    query_per_second_global: Union[Unset, float] = UNSET
    query_per_second_per_code_global: Union[Unset, "QPS"] = UNSET
    query_per_second_per_region: Union[Unset, "QPS"] = UNSET
    query_per_second_per_region_per_code: Union[Unset, "ResourceDeploymentMetricsQueryPerSecondPerRegionPerCode"] = (
        UNSET
    )
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        inference_global: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.inference_global, Unset):
            inference_global = []
            for componentsschemas_array_metric_item_data in self.inference_global:
                componentsschemas_array_metric_item = componentsschemas_array_metric_item_data.to_dict()
                inference_global.append(componentsschemas_array_metric_item)

        inference_per_region: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.inference_per_region, Unset):
            inference_per_region = self.inference_per_region.to_dict()

        inference_per_second_global: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.inference_per_second_global, Unset):
            inference_per_second_global = []
            for componentsschemas_array_metric_item_data in self.inference_per_second_global:
                componentsschemas_array_metric_item = componentsschemas_array_metric_item_data.to_dict()
                inference_per_second_global.append(componentsschemas_array_metric_item)

        inference_per_second_per_region: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.inference_per_second_per_region, Unset):
            inference_per_second_per_region = self.inference_per_second_per_region.to_dict()

        query_global = self.query_global

        query_per_code_global: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.query_per_code_global, Unset):
            query_per_code_global = self.query_per_code_global.to_dict()

        query_per_region: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.query_per_region, Unset):
            query_per_region = self.query_per_region.to_dict()

        query_per_region_per_code: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.query_per_region_per_code, Unset):
            query_per_region_per_code = self.query_per_region_per_code.to_dict()

        query_per_second_global = self.query_per_second_global

        query_per_second_per_code_global: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.query_per_second_per_code_global, Unset):
            query_per_second_per_code_global = self.query_per_second_per_code_global.to_dict()

        query_per_second_per_region: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.query_per_second_per_region, Unset):
            query_per_second_per_region = self.query_per_second_per_region.to_dict()

        query_per_second_per_region_per_code: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.query_per_second_per_region_per_code, Unset):
            query_per_second_per_region_per_code = self.query_per_second_per_region_per_code.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if inference_global is not UNSET:
            field_dict["inference_global"] = inference_global
        if inference_per_region is not UNSET:
            field_dict["inference_per_region"] = inference_per_region
        if inference_per_second_global is not UNSET:
            field_dict["inference_per_second_global"] = inference_per_second_global
        if inference_per_second_per_region is not UNSET:
            field_dict["inference_per_second_per_region"] = inference_per_second_per_region
        if query_global is not UNSET:
            field_dict["query_global"] = query_global
        if query_per_code_global is not UNSET:
            field_dict["query_per_code_global"] = query_per_code_global
        if query_per_region is not UNSET:
            field_dict["query_per_region"] = query_per_region
        if query_per_region_per_code is not UNSET:
            field_dict["query_per_region_per_code"] = query_per_region_per_code
        if query_per_second_global is not UNSET:
            field_dict["query_per_second_global"] = query_per_second_global
        if query_per_second_per_code_global is not UNSET:
            field_dict["query_per_second_per_code_global"] = query_per_second_per_code_global
        if query_per_second_per_region is not UNSET:
            field_dict["query_per_second_per_region"] = query_per_second_per_region
        if query_per_second_per_region_per_code is not UNSET:
            field_dict["query_per_second_per_region_per_code"] = query_per_second_per_region_per_code

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.metric import Metric
        from ..models.qps import QPS
        from ..models.resource_deployment_metrics_inference_per_region import (
            ResourceDeploymentMetricsInferencePerRegion,
        )
        from ..models.resource_deployment_metrics_inference_per_second_per_region import (
            ResourceDeploymentMetricsInferencePerSecondPerRegion,
        )
        from ..models.resource_deployment_metrics_query_per_region_per_code import (
            ResourceDeploymentMetricsQueryPerRegionPerCode,
        )
        from ..models.resource_deployment_metrics_query_per_second_per_region_per_code import (
            ResourceDeploymentMetricsQueryPerSecondPerRegionPerCode,
        )

        if not src_dict:
            return None
        d = src_dict.copy()
        inference_global = []
        _inference_global = d.pop("inference_global", UNSET)
        for componentsschemas_array_metric_item_data in _inference_global or []:
            componentsschemas_array_metric_item = Metric.from_dict(componentsschemas_array_metric_item_data)

            inference_global.append(componentsschemas_array_metric_item)

        _inference_per_region = d.pop("inference_per_region", UNSET)
        inference_per_region: Union[Unset, ResourceDeploymentMetricsInferencePerRegion]
        if isinstance(_inference_per_region, Unset):
            inference_per_region = UNSET
        else:
            inference_per_region = ResourceDeploymentMetricsInferencePerRegion.from_dict(_inference_per_region)

        inference_per_second_global = []
        _inference_per_second_global = d.pop("inference_per_second_global", UNSET)
        for componentsschemas_array_metric_item_data in _inference_per_second_global or []:
            componentsschemas_array_metric_item = Metric.from_dict(componentsschemas_array_metric_item_data)

            inference_per_second_global.append(componentsschemas_array_metric_item)

        _inference_per_second_per_region = d.pop("inference_per_second_per_region", UNSET)
        inference_per_second_per_region: Union[Unset, ResourceDeploymentMetricsInferencePerSecondPerRegion]
        if isinstance(_inference_per_second_per_region, Unset):
            inference_per_second_per_region = UNSET
        else:
            inference_per_second_per_region = ResourceDeploymentMetricsInferencePerSecondPerRegion.from_dict(
                _inference_per_second_per_region
            )

        query_global = d.pop("query_global", UNSET)

        _query_per_code_global = d.pop("query_per_code_global", UNSET)
        query_per_code_global: Union[Unset, QPS]
        if isinstance(_query_per_code_global, Unset):
            query_per_code_global = UNSET
        else:
            query_per_code_global = QPS.from_dict(_query_per_code_global)

        _query_per_region = d.pop("query_per_region", UNSET)
        query_per_region: Union[Unset, QPS]
        if isinstance(_query_per_region, Unset):
            query_per_region = UNSET
        else:
            query_per_region = QPS.from_dict(_query_per_region)

        _query_per_region_per_code = d.pop("query_per_region_per_code", UNSET)
        query_per_region_per_code: Union[Unset, ResourceDeploymentMetricsQueryPerRegionPerCode]
        if isinstance(_query_per_region_per_code, Unset):
            query_per_region_per_code = UNSET
        else:
            query_per_region_per_code = ResourceDeploymentMetricsQueryPerRegionPerCode.from_dict(
                _query_per_region_per_code
            )

        query_per_second_global = d.pop("query_per_second_global", UNSET)

        _query_per_second_per_code_global = d.pop("query_per_second_per_code_global", UNSET)
        query_per_second_per_code_global: Union[Unset, QPS]
        if isinstance(_query_per_second_per_code_global, Unset):
            query_per_second_per_code_global = UNSET
        else:
            query_per_second_per_code_global = QPS.from_dict(_query_per_second_per_code_global)

        _query_per_second_per_region = d.pop("query_per_second_per_region", UNSET)
        query_per_second_per_region: Union[Unset, QPS]
        if isinstance(_query_per_second_per_region, Unset):
            query_per_second_per_region = UNSET
        else:
            query_per_second_per_region = QPS.from_dict(_query_per_second_per_region)

        _query_per_second_per_region_per_code = d.pop("query_per_second_per_region_per_code", UNSET)
        query_per_second_per_region_per_code: Union[Unset, ResourceDeploymentMetricsQueryPerSecondPerRegionPerCode]
        if isinstance(_query_per_second_per_region_per_code, Unset):
            query_per_second_per_region_per_code = UNSET
        else:
            query_per_second_per_region_per_code = ResourceDeploymentMetricsQueryPerSecondPerRegionPerCode.from_dict(
                _query_per_second_per_region_per_code
            )

        resource_deployment_metrics = cls(
            inference_global=inference_global,
            inference_per_region=inference_per_region,
            inference_per_second_global=inference_per_second_global,
            inference_per_second_per_region=inference_per_second_per_region,
            query_global=query_global,
            query_per_code_global=query_per_code_global,
            query_per_region=query_per_region,
            query_per_region_per_code=query_per_region_per_code,
            query_per_second_global=query_per_second_global,
            query_per_second_per_code_global=query_per_second_per_code_global,
            query_per_second_per_region=query_per_second_per_region,
            query_per_second_per_region_per_code=query_per_second_per_region_per_code,
        )

        resource_deployment_metrics.additional_properties = d
        return resource_deployment_metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.increase_and_rate_metric import IncreaseAndRateMetric
    from ..models.metric import Metric


T = TypeVar("T", bound="Metrics")


@_attrs_define
class Metrics:
    """Metrics for resources

    Attributes:
        inference_global (Union[Unset, Any]): Historical requests for all resources globally
        inference_per_second_global (Union[Unset, list['Metric']]): Array of metrics
        query (Union[Unset, Any]): Number of requests for all resources globally
        query_per_second (Union[Unset, float]): RPS value (in last 24 hours) per location, for all resources globally
        agents (Union[Unset, IncreaseAndRateMetric]): Metrics for resources
        functions (Union[Unset, IncreaseAndRateMetric]): Metrics for resources
        models (Union[Unset, IncreaseAndRateMetric]): Metrics for resources
    """

    inference_global: Union[Unset, Any] = UNSET
    inference_per_second_global: Union[Unset, list["Metric"]] = UNSET
    query: Union[Unset, Any] = UNSET
    query_per_second: Union[Unset, float] = UNSET
    agents: Union[Unset, "IncreaseAndRateMetric"] = UNSET
    functions: Union[Unset, "IncreaseAndRateMetric"] = UNSET
    models: Union[Unset, "IncreaseAndRateMetric"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        inference_global = self.inference_global

        inference_per_second_global: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.inference_per_second_global, Unset):
            inference_per_second_global = []
            for componentsschemas_array_metric_item_data in self.inference_per_second_global:
                componentsschemas_array_metric_item = componentsschemas_array_metric_item_data.to_dict()
                inference_per_second_global.append(componentsschemas_array_metric_item)

        query = self.query

        query_per_second = self.query_per_second

        agents: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.agents, Unset):
            agents = self.agents.to_dict()

        functions: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.functions, Unset):
            functions = self.functions.to_dict()

        models: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.models, Unset):
            models = self.models.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if inference_global is not UNSET:
            field_dict["inference_global"] = inference_global
        if inference_per_second_global is not UNSET:
            field_dict["inference_per_second_global"] = inference_per_second_global
        if query is not UNSET:
            field_dict["query"] = query
        if query_per_second is not UNSET:
            field_dict["query_per_second"] = query_per_second
        if agents is not UNSET:
            field_dict["agents"] = agents
        if functions is not UNSET:
            field_dict["functions"] = functions
        if models is not UNSET:
            field_dict["models"] = models

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        from ..models.increase_and_rate_metric import IncreaseAndRateMetric
        from ..models.metric import Metric

        if not src_dict:
            return None
        d = src_dict.copy()
        inference_global = d.pop("inference_global", UNSET)

        inference_per_second_global = []
        _inference_per_second_global = d.pop("inference_per_second_global", UNSET)
        for componentsschemas_array_metric_item_data in _inference_per_second_global or []:
            componentsschemas_array_metric_item = Metric.from_dict(componentsschemas_array_metric_item_data)

            inference_per_second_global.append(componentsschemas_array_metric_item)

        query = d.pop("query", UNSET)

        query_per_second = d.pop("query_per_second", UNSET)

        _agents = d.pop("agents", UNSET)
        agents: Union[Unset, IncreaseAndRateMetric]
        if isinstance(_agents, Unset):
            agents = UNSET
        else:
            agents = IncreaseAndRateMetric.from_dict(_agents)

        _functions = d.pop("functions", UNSET)
        functions: Union[Unset, IncreaseAndRateMetric]
        if isinstance(_functions, Unset):
            functions = UNSET
        else:
            functions = IncreaseAndRateMetric.from_dict(_functions)

        _models = d.pop("models", UNSET)
        models: Union[Unset, IncreaseAndRateMetric]
        if isinstance(_models, Unset):
            models = UNSET
        else:
            models = IncreaseAndRateMetric.from_dict(_models)

        metrics = cls(
            inference_global=inference_global,
            inference_per_second_global=inference_per_second_global,
            query=query,
            query_per_second=query_per_second,
            agents=agents,
            functions=functions,
            models=models,
        )

        metrics.additional_properties = d
        return metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import math as math_operation
import operator


def math(query: str):
    """A function for performing mathematical calculations.."""
    safe_dict = {
        "abs": abs,
        "round": round,
        "min": min,
        "max": max,
        "pow": math_operation.pow,
        "sqrt": math_operation.sqrt,
        "sin": math_operation.sin,
        "cos": math_operation.cos,
        "tan": math_operation.tan,
        "pi": math_operation.pi,
        "e": math_operation.e,
    }

    # Add basic arithmetic operators
    safe_dict.update(
        {
            "+": operator.add,
            "-": operator.sub,
            "*": operator.mul,
            "/": operator.truediv,
            "**": operator.pow,
            "%": operator.mod,
        }
    )

    try:
        # Replace 'x' with '*'
        query = query.replace("x", "*")

        # Evaluate the expression in a restricted environment
        return eval(query, {"__builtins__": {}}, safe_dict)
    except Exception as e:
        raise ValueError(f"Invalid expression: {str(e)}")



from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="AgentDeploymentHistoryEvent")


@_attrs_define
class AgentDeploymentHistoryEvent:
    """Agent deployment history event

    Attributes:
        end (Union[Unset, str]): End time
        error (Union[Unset, str]): Error message
        name (Union[Unset, str]): Name of the function or agent
        parameters (Union[Unset, str]): Parameters
        start (Union[Unset, str]): Start time
        status (Union[Unset, str]): Status, eg: running, success, failed
        sub_function (Union[Unset, str]): Function used in kit if a kit was used
        took (Union[Unset, int]): Number of milliseconds it took to complete the event
        type_ (Union[Unset, str]): Type, one of function or agent
    """

    end: Union[Unset, str] = UNSET
    error: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    parameters: Union[Unset, str] = UNSET
    start: Union[Unset, str] = UNSET
    status: Union[Unset, str] = UNSET
    sub_function: Union[Unset, str] = UNSET
    took: Union[Unset, int] = UNSET
    type_: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        end = self.end

        error = self.error

        name = self.name

        parameters = self.parameters

        start = self.start

        status = self.status

        sub_function = self.sub_function

        took = self.took

        type_ = self.type_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if end is not UNSET:
            field_dict["end"] = end
        if error is not UNSET:
            field_dict["error"] = error
        if name is not UNSET:
            field_dict["name"] = name
        if parameters is not UNSET:
            field_dict["parameters"] = parameters
        if start is not UNSET:
            field_dict["start"] = start
        if status is not UNSET:
            field_dict["status"] = status
        if sub_function is not UNSET:
            field_dict["sub_function"] = sub_function
        if took is not UNSET:
            field_dict["took"] = took
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        if not src_dict:
            return None
        d = src_dict.copy()
        end = d.pop("end", UNSET)

        error = d.pop("error", UNSET)

        name = d.pop("name", UNSET)

        parameters = d.pop("parameters", UNSET)

        start = d.pop("start", UNSET)

        status = d.pop("status", UNSET)

        sub_function = d.pop("sub_function", UNSET)

        took = d.pop("took", UNSET)

        type_ = d.pop("type", UNSET)

        agent_deployment_history_event = cls(
            end=end,
            error=error,
            name=name,
            parameters=parameters,
            start=start,
            status=status,
            sub_function=sub_function,
            took=took,
            type_=type_,
        )

        agent_deployment_history_event.additional_properties = d
        return agent_deployment_history_event

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from beamlit.api.models import get_model_deployment
from beamlit.authentication import get_authentication_headers, new_client
from beamlit.common.settings import init
from beamlit.models.model_deployment import ModelDeployment
from beamlit.run import RunClient

settings = init()
client = new_client()
model_deployment: ModelDeployment = get_model_deployment.sync("gpt-4o-mini", "production", client=client)
# init_agent(client=client)
run_client = RunClient(client=client)
response = run_client.run("function", "math", settings.environment, method="POST", json={"query": "4+4"})

if response.status_code == 200:
    print(response.json())
else:
    print(response.text)
print(get_authentication_headers(settings))



import importlib
import os
import sys
import traceback
from logging import getLogger
from uuid import uuid4

from asgi_correlation_id import CorrelationIdMiddleware
from fastapi import FastAPI, Request, Response
from fastapi.responses import JSONResponse

from beamlit.common.settings import get_settings, init

from .middlewares import AccessLogMiddleware, AddProcessTimeHeader

sys.path.insert(0, os.getcwd())
sys.path.insert(0, os.path.join(os.getcwd(), "src"))


def import_module():
    settings = get_settings()
    main_module = importlib.import_module(".".join(settings.server.module.split(".")[0:-1]))
    func = getattr(main_module, settings.server.module.split(".")[-1])
    return func


settings = init()
logger = getLogger(__name__)
logger.info(f"Importing server module: {settings.server.module}")
func = import_module()
logger.info(
    f"Running server with environment {settings.environment}"
    f" on {settings.server.host}:{settings.server.port}"
)

app = FastAPI(docs_url=None, redoc_url=None)
app.add_middleware(
    CorrelationIdMiddleware,
    header_name="x-beamlit-request-id",
    generator=lambda: str(uuid4()),
)
app.add_middleware(AddProcessTimeHeader)
app.add_middleware(AccessLogMiddleware)


@app.get("/health")
async def health():
    return {"status": "ok"}


@app.post("/")
async def root(request: Request):
    settings = get_settings()
    logger = getLogger(__name__)
    try:
        body = await request.json()
        response = await func(body)
        if isinstance(response, Response):
            return response
        if type(response) is str:
            return Response(
                content=response,
                headers={"Content-Type": "text/plain"},
                media_type="text/plain",
                status_code=200,
            )
        return JSONResponse(status_code=200, content=response)
    except ValueError as e:
        content = {"error": str(e)}
        if settings.environment == "development":
            content["traceback"] = str(traceback.format_exc())
        logger.error(f"{content}")
        return JSONResponse(status_code=400, content=content)
    except Exception as e:
        content = {"error": f"Internal server error, {e}"}
        if settings.environment == "development":
            content["traceback"] = str(traceback.format_exc())
        return JSONResponse(status_code=500, content=content)
